package library;

import java.util.TreeMap;

public class Library01 {
	private TreeMap<String, Member01> members = new TreeMap<String, Member01>();
	private TreeMap<String, Bkind01> bkinds = new TreeMap<String, Bkind01>();
	private TreeMap<String, Book01> books = new TreeMap<String, Book01>();
	private int curmember = 0;
	private int curbook = 0;

	public Result01 addMember(String name){
		Result01 ret = new Result01();
		curmember++;
		String code = String.format("ME%04d",curmember);
		members.put(code, new Member01(code, name));
		ret.addInfo(new String[]{"会員番号", code});
		return ret;
	}

	public Result01 removeMember(String code){
		Result01 ret = new Result01();
		Member01 member = (Member01)members.get(code);
		if(member == null){
			ret.setCode(Result01.NO_MEMBER);
			return ret;
		}
		if(member.getNumber() != 0){
			ret.setCode(Result01.AVL_MEMBER);
			return ret;
		}
		members.remove(code);
		return ret;
	}

	public Result01 showMembers(){
		Result01 ret = new Result01();
		ret.addInfo(new String[]{"会員番号", "名前", "借りている本の数"});
		for(Member01 member: members.values()){
			String number = Integer.toString(member.getNumber());
			ret.addInfo(new String[]{member.getCode(), member.getName(), number});
		}
		return ret;
	}

	public Result01 showMember(String code){
		Result01 ret = new Result01();
		Member01 member = (Member01)members.get(code);
		if(member == null){
			ret.setCode(Result01.NO_MEMBER);
			return ret;
		}
		ret.setInfo(member.getBooksInfo());
		return ret;
	}

	public Result01 addBkind(String isbn, String title){
		Result01 ret = new Result01();
		if(bkinds.containsKey(isbn)){
			ret.setCode(Result01.DUP_BKIND);
			return ret;
		}
		bkinds.put(isbn, new Bkind01(isbn, title));
		return ret;
	}

	public Result01 removeBkind(String isbn){
		Result01 ret = new Result01();
		Bkind01 kind = (Bkind01)bkinds.get(isbn);
		if(kind == null){
			ret.setCode(Result01.NO_BKIND);
			return ret;
		}
		if(kind.getFree() != kind.getNumber()){
			ret.setCode(Result01.AVL_BKIND);
			return ret;
		}
		for(String bcode: kind.getBookCodeSet()){
			books.remove(bcode);
		}
		bkinds.remove(isbn);
		return ret;
	}

	public Result01 showBkinds(){
		Result01 ret = new Result01();
		ret.addInfo(new String[]{"ISBN番号", "タイトル", "本の全体数", "本の空き数"});
		for(Bkind01 bkind: bkinds.values()){
			String number = Integer.toString(bkind.getNumber());
			String free = Integer.toString(bkind.getFree());
			ret.addInfo(new String[]{bkind.getIsbn(), bkind.getTitle(), number, free});
		}
		return ret;
	}

	public Result01 addBook(String isbn){
		Result01 ret = new Result01();
		Bkind01 bkind = (Bkind01)bkinds.get(isbn);
		if(bkind == null){
			ret.setCode(Result01.NO_BKIND);
			return ret;
		}
		curbook++;
		String code = String.format("BK%04d", curbook);
		Book01 book = new Book01(code, bkind);
		books.put(code, book);
		bkind.addBook(book);
		ret.addInfo(new String[]{"本番号", code});
		return ret;
	}

	public Result01 removeBook(String code){
		Result01 ret = new Result01();
		Book01 book = (Book01)books.get(code);
		if(book == null){
			ret.setCode(Result01.NO_BOOK);
			return ret;
		}
		if(book.getMember() != null){
			ret.setCode(Result01.BUSY_BOOK);
			return ret;
		}
		book.getBkind().removeBook(book);
		books.remove(code);
		return ret;
	}

	public Result01 showBooks(){
		Result01 ret = new Result01();
		ret.addInfo(new String[]{"本番号", "ISBN", "タイトル", "借りている会員の番号"});
		for(Book01 book: books.values()){
			String code = book.getCode();
			String isbn = book.getBkind().getIsbn();
			String title = book.getBkind().getTitle();
			Member01 member = book.getMember();
			String mcode = (member == null ? "" : member.getCode());
			ret.addInfo(new String[]{code, isbn, title, mcode});
		}
		return ret;
	}

	public Result01 keepBook(String mcode, String bcode){
		Result01 ret = new Result01();
		Member01 member = (Member01)members.get(mcode);
		if(member == null){
			ret.setCode(Result01.NO_MEMBER);
			return ret;
		}
		Book01 book = (Book01)books.get(bcode);
		if(book == null){
			ret.setCode(Result01.NO_BOOK);
			return ret;
		}
		if(book.getMember() != null){
			ret.setCode(Result01.BUSY_BOOK);
			return ret;
		};
		book.setMember(member);
		member.keepBook(book);
		book.getBkind().keepBook();
		return ret;
	}

	public Result01 returnBook(String code){
		Result01 ret = new Result01();
		Book01 book = (Book01)books.get(code);
		if(book == null){
			ret.setCode(Result01.NO_BOOK);
			return ret;
		}
		Member01 member = book.getMember();
		if(member == null){
			ret.setCode(Result01.NOTKEEP_BOOK);
			return ret;
		}
		book.setMember(null);
		member.returnBook(book);
		book.getBkind().returnBook();
		return ret;
	}
}