package league;

import java.util.*;
import java.text.*;

public class League01 {
	private Map players = new TreeMap();
	private Map cur_matches = new TreeMap();
	private Map rec_matches = new TreeMap();
	private int curplay = 0;
	private int curmatch = 0;

	private DecimalFormat df = new DecimalFormat("0000");

	public Result01 addPlayer(String name){
		Result01 ret = new Result01();
		curplay++;
		String code = "PL" + df.format(curplay);
		players.put(code, new CPlayer01(code, name));
		ret.addInfo(new String[]{"選手番号", code});
		return ret;
	}

	public Result01 removePlayer(String code){
		Result01 ret = new Result01();
		CPlayer01 player = (CPlayer01)players.get(code);
		if(player == null){
			ret.setCode(Result01.NO_PLAYER);
			return ret;
		}
		if(player.isBusy()){
			ret.setCode(Result01.BUSY_PLAYER);
			return ret;
		}
		players.remove(code);
		return ret;
	}

	public Result01 showPlayers(){
		Result01 ret = new Result01();
		ret.addInfo(new String[]{"選手番号", "名前", "試合番号"});
		Iterator it = players.values().iterator();
		while(it.hasNext()){
			CPlayer01 player = (CPlayer01)it.next();
			Match01 match = player.getMatch();
			String matcode = match == null ? "" : match.getCode();
			ret.addInfo(new String[]{player.getCode(), player.getName(), matcode});
		}
		return ret;
	}

	public Result01 beginMatch(String code1, String code2){
		Result01 ret = new Result01();
		CPlayer01 player1 = (CPlayer01)players.get(code1);
		CPlayer01 player2 = (CPlayer01)players.get(code2);
		if(code1.equals(code2)){
			ret.setCode(Result01.DUP_PLAYER);
			return ret;
		}
		if(player1 == null || player2 == null){
			ret.setCode(Result01.NO_TWO_PLAYER);
			return ret;
		}
		if(player1.isBusy() || player2.isBusy()){
			ret.setCode(Result01.BUSY_EITHER_PLAYER);
			return ret;
		}
		curmatch++;
		String code = "MT" + df.format(curmatch);
		Match01 match = new Match01(code, player1, player2);
		player1.setMatch(match);
		player2.setMatch(match);
		cur_matches.put(code, match);
		ret.addInfo(new String[]{"試合番号", code});
		return ret;
	}

	public Result01 endMatch(String code, int score1, int score2){
		Result01 ret = new Result01();
		Match01 match = (Match01)cur_matches.get(code);
		CPlayer01 player1 = (CPlayer01)match.getPlayer(0);
		CPlayer01 player2 = (CPlayer01)match.getPlayer(1);
		match.setPlayer(player1.getRPlayer(score1), player2.getRPlayer(score2));
		cur_matches.remove(code);
		rec_matches.put(code, match);
		return ret;
	}

	public Result01 showCurMatches(){
		Result01 ret = new Result01();
		ret.addInfo(new String[]{"試合番号", "選手番号", "名前", "選手番号", "名前"});
		Iterator it = cur_matches.values().iterator();
		while(it.hasNext()){
			Match01 match = (Match01)it.next();
			Player01 player1 = match.getPlayer(0);
			Player01 player2 = match.getPlayer(1);
			ret.addInfo(
				new String[]{
					match.getCode(),
					player1.getCode(),
					player1.getName(),
					player2.getCode(),
					player2.getName()
				}
			);
		}
		return ret;
	}

	public Result01 showRecMatches(){
		Result01 ret = new Result01();
		ret.addInfo(
			new String[]{
				"試合番号",
				"選手番号",
				"スコア",
				"名前",
				"選手番号",
				"名前",
				"スコア"
			}
		);
		Iterator it = rec_matches.values().iterator();
		while(it.hasNext()){
			Match01 match = (Match01)it.next();
			RPlayer01 player1 = (RPlayer01)match.getPlayer(0);
			RPlayer01 player2 = (RPlayer01)match.getPlayer(1);
			ret.addInfo(
				new String[]{
					match.getCode(),
					player1.getCode(),
					player1.getName(),
					Integer.toString(player1.getScore()),
					player2.getCode(),
					player2.getName(),
					Integer.toString(player2.getScore())
				}
			);
		}
		return ret;
	}

	public Result01 getMatch(String code){
		Result01 ret = new Result01();
		Match01 match = (Match01)cur_matches.get(code);
		if(match == null){
			ret.setCode(Result01.NO_MATCH);
			return ret;
		}
		Player01 player1 = match.getPlayer(0);
		Player01 player2 = match.getPlayer(1);
		ret.addInfo(new String[]{player1.getCode() + " " + player1.getName()});
		ret.addInfo(new String[]{player2.getCode() + " " + player2.getName()});
		return ret;
	}
}
